import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
/**
 * The `DoubleClickZoomHandler` allows the user to zoom the map at a point by
 * double clicking or double tapping.
 */
export default class DoubleClickZoomHandler {
  /**
   * @private
   */
  constructor(clickZoom, TapZoom) {
    _defineProperty(this, "clickZoom", void 0);
    _defineProperty(this, "tapZoom", void 0);
    this.clickZoom = clickZoom;
    this.tapZoom = TapZoom;
  }

  /**
   * Enables the "double click to zoom" interaction.
   *
   * @example
   * map.doubleClickZoom.enable();
   */
  enable() {
    this.clickZoom.enable();
    this.tapZoom.enable();
  }

  /**
   * Disables the "double click to zoom" interaction.
   *
   * @example
   * map.doubleClickZoom.disable();
   */
  disable() {
    this.clickZoom.disable();
    this.tapZoom.disable();
  }

  /**
   * Returns a Boolean indicating whether the "double click to zoom" interaction is enabled.
   *
   * @returns {boolean} `true` if the "double click to zoom" interaction is enabled.
   */
  isEnabled() {
    return this.clickZoom.isEnabled() && this.tapZoom.isEnabled();
  }

  /**
   * Returns a Boolean indicating whether the "double click to zoom" interaction is active, i.e. currently being used.
   *
   * @returns {boolean} `true` if the "double click to zoom" interaction is active.
   */
  isActive() {
    return this.clickZoom.isActive() || this.tapZoom.isActive();
  }
}