"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;
var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));
var _point = _interopRequireDefault(require("../../geo/point"));
var _dom = _interopRequireDefault(require("../../utils/dom"));
var _event = require("./event");
// @ts-ignore

class MapTouchEvent extends _event.Event {
  /**
   * @private
   */
  constructor(type, map, originalEvent) {
    const touches = type === 'touchend' ? originalEvent.changedTouches : originalEvent.touches;
    const points = _dom.default.touchPos(map.getCanvasContainer(), touches);
    const lngLats = points.map(t => map.unproject(t));
    const point = points.reduce((prev, curr, i, arr) => {
      return prev.add(curr.div(arr.length));
    }, new _point.default(0, 0));
    const lngLat = map.unproject(point);
    super(type, {
      points,
      point,
      lngLats,
      lngLat,
      originalEvent
    });
    /**
     * The event type.
     */
    /**
     * The `Map` object that fired the event.
     */
    (0, _defineProperty2.default)(this, "target", void 0);
    /**
     * The DOM event which caused the map event.
     */
    (0, _defineProperty2.default)(this, "originalEvent", void 0);
    /**
     * The geographic location on the map of the center of the touch event points.
     */
    (0, _defineProperty2.default)(this, "lngLat", void 0);
    /**
     * The pixel coordinates of the center of the touch event points, relative to the map and measured from the top left
     * corner.
     */
    (0, _defineProperty2.default)(this, "point", void 0);
    /**
     * The array of pixel coordinates corresponding to a
     * [touch event's `touches`](https://developer.mozilla.org/en-US/docs/Web/API/TouchEvent/touches) property.
     */
    (0, _defineProperty2.default)(this, "points", void 0);
    /**
     * The geographical locations on the map corresponding to a
     * [touch event's `touches`](https://developer.mozilla.org/en-US/docs/Web/API/TouchEvent/touches) property.
     */
    (0, _defineProperty2.default)(this, "lngLats", void 0);
    /**
     * `true` if `preventDefault` has been called.
     * @private
     */
    (0, _defineProperty2.default)(this, "defaultPrevented", void 0);
    this.defaultPrevented = false;
  }

  /**
   * Prevents subsequent default processing of the event by the map.
   *
   * Calling this method will prevent the following default map behaviors:
   *
   *   * On `touchstart` events, the behavior of {@link DragPanHandler}
   *   * On `touchstart` events, the behavior of {@link TouchZoomRotateHandler}
   *
   */
  preventDefault() {
    this.defaultPrevented = true;
  }
}
exports.default = MapTouchEvent;