"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.bezier = bezier;
exports.cancel = void 0;
exports.clamp = clamp;
exports.ease = void 0;
exports.extend = extend;
exports.interpolate = interpolate;
exports.now = void 0;
exports.pick = pick;
exports.prefersReducedMotion = prefersReducedMotion;
exports.raf = void 0;
exports.renderframe = renderframe;
exports.wrap = wrap;
var _unitbezier = _interopRequireDefault(require("@mapbox/unitbezier"));
// @ts-ignore

let reducedMotionQuery;
function wrap(n, min, max) {
  const d = max - min;
  const w = ((n - min) % d + d) % d + min;
  return w === min ? max : w;
}
function clamp(n, min, max) {
  return Math.min(max, Math.max(min, n));
}
function interpolate(a, b, t) {
  return a * (1 - t) + b * t;
}
function bezier(p1x, p1y, p2x, p2y) {
  const bez = new _unitbezier.default(p1x, p1y, p2x, p2y);
  return t => {
    return bez.solve(t);
  };
}
const ease = exports.ease = bezier(0.25, 0.1, 0.25, 1);
function prefersReducedMotion() {
  var _reducedMotionQuery;
  // Lazily initialize media query
  if (reducedMotionQuery == null && typeof window !== 'undefined' && window.matchMedia) {
    // @ts-ignore
    reducedMotionQuery = window.matchMedia('(prefers-reduced-motion: reduce)');
  }
  return (_reducedMotionQuery = reducedMotionQuery) === null || _reducedMotionQuery === void 0 ? void 0 : _reducedMotionQuery.matches;
}
function pick(src, properties) {
  const result = {};
  for (const name of properties) {
    if (name in src) {
      result[name] = src[name];
    }
  }
  return result;
}
const now = exports.now = window.performance && window.performance.now ? window.performance.now.bind(window.performance) : Date.now.bind(Date);
const raf = exports.raf = window.requestAnimationFrame ||
// @ts-ignore
window.mozRequestAnimationFrame ||
// @ts-ignore
window.webkitRequestAnimationFrame ||
// @ts-ignore
window.msRequestAnimationFrame;
const cancel = exports.cancel = window.cancelAnimationFrame ||
// @ts-ignore
window.mozCancelAnimationFrame ||
// @ts-ignore
window.webkitCancelAnimationFrame ||
// @ts-ignore
window.msCancelAnimationFrame;
function renderframe(fn) {
  const frame = raf(fn);
  return {
    cancel: () => cancel(frame)
  };
}
function extend(dest, ...sources) {
  for (const src of sources) {
    for (const k in src) {
      if (src[k] !== undefined) {
        dest[k] = src[k];
      }
    }
  }
  return dest;
}